@@ -1,8 +1,9 @@
 /*
  * virtual private network daemon (vpnd)
  *
- * cryptographic stuff (c) 1999 Andreas Steinmetz, astmail@yahoo.com
+ * cryptographic stuff (c) 1999 Andreas Steinmetz, ast@domdv.de
  * other code (c) 1999 D.O.M. Datenverarbeitung GmbH, author Andreas Steinmetz
+ * (c) 2001 Andreas Steinmetz
  *
  * License:
  * This code is in the public domain (*) under the GNU public license.
@@ -26,6 +27,8 @@
  * getrandom
  *
  * input:  len    - the amount of random data to collect
+ *	   mode   - 0 = fail if insufficient data
+ *		    1 = block and continue reading until request complete
  *	   anchor - pointer to vpnd global data
  *
  * output: out - 64 bytes of random data
@@ -36,10 +39,14 @@
  * This procedure gathers and returns 64 bytes of random data.
  */
 
-int getrandom(WORD08 *out,WORD32 len,VPN *anchor)
+int getrandom(WORD08 *out,WORD32 len,WORD32 mode,VPN *anchor)
 {
-	int fd;	/* file descriptor of random device */
-
+	int fd;			/* file descriptor of random device	*/
+	int n;			/* count of bytes read			*/
+#ifdef SunOS
+	struct sockaddr_un s;	/* unix domain socket			*/
+        WORD08 msg[2];		/* request message to prngd		*/
+#endif
 
 	/* debug message */
 
@@ -48,14 +55,52 @@ int getrandom(WORD08 *out,WORD32 len,VPN *anchor)
 	/* open random device (no block if insufficient entropy),
 	   handle errors */
 
+#ifdef SunOS
+	if(len>255)RETURN("len",0);
+	memset(&s,0,sizeof(s));
+	if(sizeof(s.sun_path)<=strlen(anchor->randomdev))RETURN("path",0);
+	strcpy(s.sun_path,anchor->randomdev);
+	s.sun_family=AF_UNIX;
+	if((fd=socket(AF_UNIX,SOCK_STREAM,0))==-1)RETURN("socket",0);
+	if(connect(fd,(struct sockaddr *)(&s),sizeof(s))<0)
+	{
+		close(fd);
+		RET("connect",0);
+	}
+
+	/* SunOS special: blocking request so we always wait for random
+	   data to be available */
+
+	msg[0]=2;
+	msg[1]=(WORD08)(len);
+	if(write(fd,msg,2)!=2)
+	{
+		close(fd);
+		RET("write",0);
+	}
+#else
 	if((fd=open(anchor->randomdev,O_RDONLY))<0)RETURN("open",0);
+#endif
 
 	/* read random data, handle errors */
 
-	if(read(fd,out,len)!=len)
+	if(!mode)
 	{
-		close(fd);
-		RET("read",0);
+		if(read(fd,out,len)!=len)
+		{
+			close(fd);
+			RET("read",0);
+		}
+	}
+	else for(n=0;len;)
+	{
+		if((n=read(fd,out,len))<=0)	
+		{
+			close(fd);
+			RET("read",0);
+		}
+		out+=n;
+		len-=n;
 	}
 
 	/* close random device */
@@ -292,22 +337,22 @@ void decrypt_cfb_64_8(WORD08 *data,WORD32 total,VPN *anchor)
  *
  * This procedure sends data encypted to the remote side.
  * It wraps the data with integrity checking information.
- * A maximum of BLOCKSIZE bytes can be sent. If the length of
+ * A maximum of MAXBLOCK bytes can be sent. If the length of
  * the data to be sent is 0, new key exchange is done.
  */
 
-#ifndef CRYPTOX86
-
 int datasend(register WORD08 *data,register WORD32 length,VPN *anchor)
 {
 	register WORD32 i;			/* counter/index	*/
-	WORD08 bfr[22+MAXBLOCK+(MAXBLOCK/2)];	/* work buffer		*/
-	WORD08 *hdr=bfr+20-anchor->mac;		/* data header		*/
-	WORD08 *tmp=bfr+22;			/* compression/data bfr	*/
+	WORD08 bfr[24+MAXBLOCK+(MAXBLOCK/2)];	/* work buffer		*/
+	WORD08 *hdr=bfr+22-anchor->mac;		/* data header		*/
+	WORD08 *tmp=bfr+24;			/* compression/data bfr	*/
 						/* new key and iv memory*/
 #ifdef COMPRESS
-	WORD32 tmplen;				/* used for compression	*/
+	unsigned long tmplen;			/* used for compression	*/
 #endif
+	WORD32 nopkt=0;				/* packetize disable	*/
+	WORD32 delta=0;				/* rtp header space	*/
 
 
 	/* debug message */
@@ -336,7 +381,7 @@ int datasend(register WORD08 *data,register WORD32 length,VPN *anchor)
 		{
 			/* get 80 random bytes */
 
-			if(!getrandom(tmp,BLOWFISH_BLOCKSIZE+BLOWFISH_MAXKEY,
+			if(!getrandom(tmp,BLOWFISH_BLOCKSIZE+BLOWFISH_MAXKEY,0,
 				anchor))GOTO("getrandom",keyerr);
 
 			/* encrypt 80 bytes in ECB mode (you never know) */
@@ -390,7 +435,7 @@ int datasend(register WORD08 *data,register WORD32 length,VPN *anchor)
 		{
 			/* get 8 random bytes */
 
-			if(!getrandom(tmp,BLOWFISH_BLOCKSIZE,anchor))
+			if(!getrandom(tmp,BLOWFISH_BLOCKSIZE,0,anchor))
 				GOTO("getrandom",keyerr);
 
 			/* encrypt 8 bytes in ECB mode (you never know) */
@@ -451,9 +496,24 @@ keyerr:		for(i=0;i<BLOWFISH_BLOCKSIZE+BLOWFISH_MAXKEY;i++)
 		return 0;
 	}
 
-	/* signal error if maximum length exceeded */
+	/* if maximum length exceeded */
 
-	if(length>MAXBLOCK)RET("illegal length",0);
+	if(length>MAXBLOCK)
+	{
+		/* handle special case where packetize mode cannot be used */
+
+		if(length==-BLOWFISH_BLOCKSIZE)
+		{
+			nopkt=1;
+			length=BLOWFISH_BLOCKSIZE;
+		}
+
+		/* in all other cases signal error but respect two additional
+		   bytes in packetize mode that are stripped */
+
+		else if(!anchor->pktmode||length>MAXBLOCK+2)
+			RET("illegal length",0);
+	}
 
 	/* debug only */
 
@@ -467,6 +527,31 @@ keyerr:		for(i=0;i<BLOWFISH_BLOCKSIZE+BLOWFISH_MAXKEY;i++)
 	}
 #endif
 
+	/* if in packetize mode and allowed to do so */
+
+	if(!nopkt&&anchor->pktmode)
+	{
+		/* copy format information to length high byte */
+
+		hdr[anchor->mac+1]=data[0]&0x70;
+
+		/* if simple checksum for rtp packets is selected and we
+		   do have a rtp stream calculate the delta between
+		   selected checksum size and simple checksum size */
+
+		if(anchor->smallrtp&&hdr[anchor->mac+1]>=(RTP_COMP<<4))
+			delta=anchor->mac-2;
+
+		/* strip packet length and format information from data */
+
+		data+=2;
+		length-=2;
+	}
+
+	/* reset length high byte if not in packetize mode or not allowed */
+
+	else hdr[anchor->mac+1]=0;
+
 #ifdef COMPRESS
 
 	/* set up size of compression buffer */
@@ -509,7 +594,7 @@ keyerr:		for(i=0;i<BLOWFISH_BLOCKSIZE+BLOWFISH_MAXKEY;i++)
 		/* create length header with MSB set */
 
 		hdr[anchor->mac]=(WORD08)(length);
-		hdr[anchor->mac+1]=((WORD08)(length>>8))|0x80;
+		hdr[anchor->mac+1]=((WORD08)((length>>8)&0xf))|0x80;
 	}
 
 	/* in case of no compression */
@@ -523,7 +608,7 @@ keyerr:		for(i=0;i<BLOWFISH_BLOCKSIZE+BLOWFISH_MAXKEY;i++)
 		/* create regular length header */
 
 		hdr[anchor->mac]=(WORD08)(length);
-		hdr[anchor->mac+1]=(WORD08)(length>>8);
+		hdr[anchor->mac+1]|=(WORD08)(length>>8);
 	}
 #else
 	/* copy data to buffer */
@@ -533,16 +618,52 @@ keyerr:		for(i=0;i<BLOWFISH_BLOCKSIZE+BLOWFISH_MAXKEY;i++)
 	/* create length header */
 
 	hdr[anchor->mac]=(WORD08)(length);
-	hdr[anchor->mac+1]=(WORD08)(length>>8);
+	hdr[anchor->mac+1]|=(WORD08)((length&0xf)>>8);
 #endif
 
-	/* create checksum header */
+	/* if there is room for rtp stream transmission instead of a
+	   large checksum in the header and if the payload is large enough
+	   to fill the room and still leave some payload byte(s) */
 
-	(anchor->macproc)(tmp,length,hdr,(void *)(&(anchor->hmkey)));
+	if(delta&&length>delta)
+	{
+		/* memorize length header */
+
+		bfr[0]=hdr[anchor->mac];
+		bfr[1]=hdr[anchor->mac+1];
+
+		/* shift header start to fit simple checksum */
+
+		hdr+=delta;
+
+		/* create simple checksum of payload */
+
+		checksum(tmp,length,hdr,NULL);
+
+		/* move data to old length header location */
+
+		hdr[2]=hdr[anchor->mac];
+		hdr[3]=hdr[anchor->mac+1];
+
+		/* set up length header in place of moved data */
+
+		hdr[anchor->mac]=bfr[0];
+		hdr[anchor->mac+1]=bfr[1];
+
+		/* reduce data length with moved to header data amount
+		   and adjust start of data accordingly */
+
+		tmp+=delta;
+		length-=delta;
+	}
+
+	/* otherwise create standard checksum header */
+
+	else (anchor->macproc)(tmp,length,hdr,(void *)(&(anchor->hmkey)));
 
 	/* debug message (only the first four bytes of checksum for hmac) */
 
-	DEBUG3(4,"length=%04lx chksum=%04lx\n",
+	DEBUG3(4,"length=%04x chksum=%04x\n",
 		((WORD32)(hdr[anchor->mac+1])<<8)|(WORD32)(hdr[anchor->mac]),
 		(anchor->hmacmode?((WORD32)(hdr[3])<<24)|
 		(WORD32)(hdr[2]<<16):0)|
@@ -569,8 +690,6 @@ keyerr:		for(i=0;i<BLOWFISH_BLOCKSIZE+BLOWFISH_MAXKEY;i++)
 	return 1;
 }
 
-#endif
-
 /*
  * datarecv
  *
@@ -589,8 +708,6 @@ keyerr:		for(i=0;i<BLOWFISH_BLOCKSIZE+BLOWFISH_MAXKEY;i++)
  * received key.
  */
 
-#ifndef CRYPTOX86
-
 int datarecv(register WORD08 *data,WORD32 max,VPN *anchor)
 {
 	WORD08 hdr[22];				/* data header		*/
@@ -598,7 +715,7 @@ int datarecv(register WORD08 *data,WORD32 max,VPN *anchor)
 	register WORD32 i;			/* counter/index	*/
 	register WORD32 length;			/* data length		*/
 #ifdef COMPRESS
-	WORD32 tmplen=max;			/* used for compression	*/
+	unsigned long tmplen=max;		/* used for compression	*/
 	WORD08 tmp[MAXBLOCK+(MAXBLOCK/2)];	/* compression buffer	*/
 						/* new key and iv 	*/
 	WORD08 *newkey=tmp;
@@ -606,6 +723,8 @@ int datarecv(register WORD08 *data,WORD32 max,VPN *anchor)
 						/* new key and iv 	*/
 	WORD08 newkey[BLOWFISH_BLOCKSIZE+BLOWFISH_MAXKEY];
 #endif
+	WORD32 nopkt=0;				/* packetize disable	*/
+	WORD32 delta=0;				/* rtp header space	*/
 
 
 	/* debug message */
@@ -622,7 +741,7 @@ int datarecv(register WORD08 *data,WORD32 max,VPN *anchor)
 
 	/* debug message (only first four bytes for hmac) */
 
-	DEBUG3(4,"length=%04lx chksum=%04lx\n",
+	DEBUG3(4,"length=%04x chksum=%04x\n",
 		((WORD32)(hdr[anchor->mac+1])<<8)|(WORD32)(hdr[anchor->mac]),
 		(anchor->hmacmode?((WORD32)(hdr[3])<<24)|(WORD32)(hdr[2]<<16):0)|
 		((WORD32)(hdr[1])<<8)|(WORD32)(hdr[0]));
@@ -659,7 +778,7 @@ int datarecv(register WORD08 *data,WORD32 max,VPN *anchor)
 
 			/* debug message */
 
-			DEBUG3(4,"length=%04lx chksum=%04lx\n",
+			DEBUG3(4,"length=%04x chksum=%04x\n",
 				((WORD32)(hdr[anchor->mac+1])<<8)|
 					(WORD32)(hdr[anchor->mac]),
 				(anchor->hmacmode?((WORD32)(hdr[3])<<24)|
@@ -696,7 +815,7 @@ int datarecv(register WORD08 *data,WORD32 max,VPN *anchor)
 
 			/* debug message */
 
-			DEBUG3(4,"length=%04lx chksum=%04lx\n",
+			DEBUG3(4,"length=%04x chksum=%04x\n",
 				((WORD32)(hdr[anchor->mac+1])<<8)|
 					(WORD32)(hdr[anchor->mac]),
 				(anchor->hmacmode?((WORD32)(hdr[3])<<24)|
@@ -746,14 +865,36 @@ keyerr:		for(i=0;i<BLOWFISH_BLOCKSIZE+BLOWFISH_MAXKEY;i++)
 
 	/* restore data length */
 
-#ifdef COMPRESS
-	length=(((WORD32)((hdr[anchor->mac+1]&0x7f)))<<8)|((WORD32)(hdr[anchor->mac]));
-#else
-	length=(((WORD32)(hdr[anchor->mac+1]))<<8)|((WORD32)(hdr[anchor->mac]));
-#endif
+	length=(((WORD32)((hdr[anchor->mac+1]&0xf)))<<8)|
+		((WORD32)(hdr[anchor->mac]));
+
+	/* handle special case where packetize cannot be used */
+
+	if(max==-BLOWFISH_BLOCKSIZE&&length==BLOWFISH_BLOCKSIZE)nopkt=1;
+
 	/* signal error if length exceeds buffer size */
 
-	if(length>max)RET("illegal length",0);
+	else if(length>max)RET("illegal length",0);
+
+	/* if in packetize mode and allowed to do so */
+
+	if(!nopkt&&anchor->pktmode)
+	{
+		/* copy format information to packet header */
+
+		*data=hdr[anchor->mac+1]&0x70;
+
+		/* skip packet header */
+
+		data+=2;
+
+		/* if simple checksum for rtp packets is selected and we
+		   have a rtp packet and the packet is large enough
+		   set up the amount of data bytes in checksum location */
+
+		if(anchor->smallrtp&&hdr[anchor->mac+1]>=(RTP_COMP<<4))
+			if((delta=anchor->mac-2))if(length<=delta)delta=0;
+	}
 
 #ifdef COMPRESS
 
@@ -767,15 +908,30 @@ keyerr:		for(i=0;i<BLOWFISH_BLOCKSIZE+BLOWFISH_MAXKEY;i++)
 
 		/* read expected amount of data, signal errors */
 
-		if(!dorecv(tmp,length,anchor))RET("dorecv data",0);
+		if(!dorecv(tmp+delta,length-delta,anchor))RET("dorecv data",0);
 
 		/* decrypt received data if not in plaintext mode */
 
-		decrypt_cfb_64_8(tmp,length,anchor);
+		decrypt_cfb_64_8(tmp+delta,length-delta,anchor);
 
-		/* calculate checksum of read data */
+		/* if there is rtp packet data in the header */
 
-		(anchor->macproc)(tmp,length,sum,
+		if(delta)
+		{
+			/* copy the packet data to the packet body */
+
+			memcpy(tmp,hdr+4,delta-2);
+			tmp[delta-2]=hdr[2];
+			tmp[delta-1]=hdr[3];
+
+			/* calculate simple checksum */
+
+			checksum(tmp,length,sum,NULL);
+		}
+
+		/* otherwise calculate standard checksum of read data */
+
+		else (anchor->macproc)(tmp,length,sum,
 			(void *)(&(anchor->hmkey)));
 
 		/* decompress data, signal errors */
@@ -794,30 +950,74 @@ keyerr:		for(i=0;i<BLOWFISH_BLOCKSIZE+BLOWFISH_MAXKEY;i++)
 	{
 		/* read expected amount of data, signal errors */
 
-		if(!dorecv(data,length,anchor))RET("dorecv data",0);
+		if(!dorecv(data+delta,length-delta,anchor))RET("dorecv data",0);
 
 		/* decrypt received data if not in plaintext mode */
 
-		decrypt_cfb_64_8(data,length,anchor);
+		decrypt_cfb_64_8(data+delta,length-delta,anchor);
 
-		/* calculate checksum of read data */
+		/* if there is rtp packet data in the header */
 
-		(anchor->macproc)(data,length,sum,
+		if(delta)
+		{
+			/* copy the packet data to the packet body */
+
+			memcpy(data,hdr+4,delta-2);
+			data[delta-2]=hdr[2];
+			data[delta-1]=hdr[3];
+
+			/* calculate simple checksum */
+
+			checksum(data,length,sum,NULL);
+		}
+
+		/* otherwise calculate standard checksum of read data */
+
+		else (anchor->macproc)(data,length,sum,
 			(void *)(&(anchor->hmkey)));
 	}
 #else
 	/* read expected amount of data, signal errors */
 
-	if(!dorecv(data,length,anchor))RET("dorecv data",0);
+	if(!dorecv(data+delta,length-delta,anchor))RET("dorecv data",0);
 
 	/* decrypt received data if not in plaintext mode */
 
-	decrypt_cfb_64_8(data,length,anchor);
+	decrypt_cfb_64_8(data+delta,length-delta,anchor);
 
-	/* calculate checksum of read data */
+	/* if there is rtp packet data in the header */
 
-	(anchor->macproc)(data,length,sum,(void *)(&(anchor->hmkey)));
+	if(delta)
+	{
+		/* copy the packet data to the packet body */
+
+		memcpy(data,hdr+4,delta-2);
+		data[delta-2]=hdr[2];
+		data[delta-1]=hdr[3];
+
+		/* calculate simple checksum */
+
+		checksum(data,length,sum,NULL);
+	}
+
+	/* otherwise calculate standard checksum of read data */
+
+	else (anchor->macproc)(data,length,sum,(void *)(&(anchor->hmkey)));
 #endif
+
+	/* if in packetize mode and allowed to do so */
+
+	if(!nopkt&&anchor->pktmode)
+	{
+		/* copy format info to packet header, regenerate packet header
+		   length and add packet header to returned data */
+
+		data-=2;
+		data[0]|=(WORD08)((--length>>8)&0xf);
+		data[1]=(WORD08)(length);
+		length+=3;
+	}
+
 	/* debug message */
 
 #ifdef DEBUG
@@ -839,9 +1039,11 @@ keyerr:		for(i=0;i<BLOWFISH_BLOCKSIZE+BLOWFISH_MAXKEY;i++)
 	}
 #endif
 
-	/* in case of checksum mismatch signal error */
+	/* in case of checksum mismatch signal error (take rtp packet simple
+	   checksums into account) */
 
-	for(i=0;i<anchor->mac;i++)if(hdr[i]!=sum[i])RET("chksum error",0);
+	for(i=0;i<(delta?2:anchor->mac);i++)if(hdr[i]!=sum[i])
+		RET("chksum error",0);
 
 	/* debug message */
 
@@ -852,8 +1054,6 @@ keyerr:		for(i=0;i<BLOWFISH_BLOCKSIZE+BLOWFISH_MAXKEY;i++)
 	return length+1;
 }
 
-#endif
-
 /*
  * encrypt_cbc
  *
@@ -959,7 +1159,7 @@ void genwhite(WORD08 *src,VPN *anchor)
 
 	/* generate key schedule from master key */
 
-	blowfish_key_schedule(src,anchor->keysize,sched);
+	blowfish_key_schedule(src,BLOWFISH_MAXKEY,sched);
 
 	/* preset the whitening buffer with well known values */
 
@@ -1056,9 +1256,15 @@ int writemaster(int sync,VPN *anchor)
 
 	/* open key file, handle errors */
 
+#ifdef O_SYNC
 	if((f=open(anchor->keyfile,
 		sync?(O_WRONLY|O_CREAT|O_EXCL|O_SYNC):O_WRONLY,0400))==-1)
 		RETURN("open",0);
+#else
+	if((f=open(anchor->keyfile,
+		sync?(O_WRONLY|O_CREAT|O_EXCL):O_WRONLY,0400))==-1)
+		RETURN("open",0);
+#endif
 
 	/* write key file, handle errors: note that so synchronized
 	   access or fsync() would be required here we anyway write
@@ -1147,13 +1353,18 @@ int genmaster(VPN *anchor)
 
 	if(getuid()||getgid())
 	{
-		printf("You must be root and group root to create a key\n");
+		printf("You must be root and group root to create a key.\n");
 		return 1;
 	}
 
+	/* warn user that this could be lengthy */
+
+	printf("This may take a while. Please continue working in another " \
+		"session to allow\ngathering of random data.\n");
+
 	/* get 72 random bytes */
 
-	if(!getrandom(ptr,BLOWFISH_MAXKEY,anchor))goto rnderr;
+	if(!getrandom(ptr,BLOWFISH_MAXKEY,1,anchor))goto rnderr;
 
 	/* generate key schedule from random data */
 
@@ -1161,7 +1372,7 @@ int genmaster(VPN *anchor)
 
 	/* get another 72 random bytes */
 
-	if(!getrandom(master1,BLOWFISH_MAXKEY,anchor))goto rnderr;
+	if(!getrandom(master1,BLOWFISH_MAXKEY,1,anchor))goto rnderr;
 
 	/* encrypt 72 bytes in ECB mode */
 
@@ -1203,11 +1414,12 @@ int genmaster(VPN *anchor)
 
 		/* get local iv */
 
-		if(!getrandom(rkey+IV0,BLOWFISH_BLOCKSIZE,anchor))goto rnderr;
+		if(!getrandom(rkey+IV0,BLOWFISH_BLOCKSIZE,1,anchor))goto rnderr;
 
 		/* get remote iv */
 
-		if(!getrandom(secret+IV0,BLOWFISH_BLOCKSIZE,anchor))goto rnderr;
+		if(!getrandom(secret+IV0,BLOWFISH_BLOCKSIZE,1,anchor))
+			goto rnderr;
 
 		/* duplicate ivs */
 
@@ -1216,7 +1428,7 @@ int genmaster(VPN *anchor)
 
 		/* generate key schedule from default password */
 
-		blowfish_key_schedule("vpnd",4,sched);
+		blowfish_key_schedule((unsigned char *)"vpnd",4,sched);
 
 		/* encrypt local master secret in CBC mode and duplicate it */
 
@@ -1242,7 +1454,7 @@ int genmaster(VPN *anchor)
 
 		/* get remaining key identifier */
 
-		if(!getrandom(ptr+4,20,anchor))
+		if(!getrandom(ptr+4,20,1,anchor))
 		{
 rnderr:			printf("Failed to gather random data\n");
 			return 2;
@@ -1268,7 +1480,11 @@ rnderr:			printf("Failed to gather random data\n");
 
 		/* print message */
 
-		printf("New key file %s/%s created.\n",getcwd(NULL,0),
+		if(anchor->keyfile[0]!='/')
+			printf("New key file %s/%s created.\n",getcwd(NULL,0),
+			anchor->keyfile);
+		else
+			printf("New key file %s created.\n",
 			anchor->keyfile);
 
 		/* prepare writing of remote key to file */
@@ -1291,7 +1507,12 @@ crterr:		printf("Can't create %s\n",anchor->keyfile);
 
 	/* print message */
 
-	printf("New key file %s/%s created.\n",getcwd(NULL,0),anchor->keyfile);
+	if(anchor->keyfile[0]!='/')
+		printf("New key file %s/%s created.\n",getcwd(NULL,0),
+			anchor->keyfile);
+	else
+		printf("New key file %s created.\n",
+			anchor->keyfile);
 
 	/* debug message */
 
@@ -1309,7 +1530,7 @@ crterr:		printf("Can't create %s\n",anchor->keyfile);
  *
  * return: 0 - success
  *	   1 - send/receive of new key and server iv failed
- *	   2 - send/receive of iv failed
+ *	   2 - send/receive of client iv failed
  *	   3 - master key file read error
  *	   4 - master key file version not supported
  *	   5 - extended information receive failed
@@ -1484,7 +1705,7 @@ int cryptosetup(VPN *anchor)
 
 		/* create key schedule from default password */
 
-		blowfish_key_schedule("vpnd",4,anchor->xsched);
+		blowfish_key_schedule((unsigned char *)"vpnd",4,anchor->xsched);
 
 		/* receive local iv from peer, handle errors */
 
@@ -1511,8 +1732,8 @@ int cryptosetup(VPN *anchor)
 
 	/* convert local and remote ip to numbers */
 
-	src=(WORD32)ntohl(getaddr(anchor->local_ip));
-	dst=(WORD32)ntohl(getaddr(anchor->remote_ip));
+	src=(WORD32)ntohl(getaddr((unsigned char *)(anchor->local_ip)));
+	dst=(WORD32)ntohl(getaddr((unsigned char *)(anchor->remote_ip)));
 
 	/* copy local and remote ips to initial key according to client
 	   or server mode */
@@ -1600,7 +1821,7 @@ int cryptosetup(VPN *anchor)
 
 		/* get new random local iv */
 
-		if(!getrandom(xbfr,BLOWFISH_BLOCKSIZE,anchor))
+		if(!getrandom(xbfr,BLOWFISH_BLOCKSIZE,0,anchor))
 			RET("getrandom",11);
 
 		/* encrypt current master secret */
@@ -1619,11 +1840,13 @@ int cryptosetup(VPN *anchor)
 
 		/* send new local iv to peer, handle errors */
 
-		if(!datasend(xbfr,BLOWFISH_BLOCKSIZE,anchor))RET("datasend",12);
+		if(!datasend(xbfr,-BLOWFISH_BLOCKSIZE,anchor))
+			RET("datasend",12);
 
 		/* receive new remote iv from peer, handle errors */
 
-		if(!datarecv(xbfr,BLOWFISH_BLOCKSIZE,anchor))RET("datarecv",13);
+		if(!datarecv(xbfr,-BLOWFISH_BLOCKSIZE,anchor))
+			RET("datarecv",13);
 
 		/* store remote iv */
 
@@ -1639,11 +1862,13 @@ int cryptosetup(VPN *anchor)
 
 		/* send result to peer, handle errors */
 
-		if(!datasend(xbfr,BLOWFISH_BLOCKSIZE,anchor))RET("datasend",14);
+		if(!datasend(xbfr,-BLOWFISH_BLOCKSIZE,anchor))
+			RET("datasend",14);
 
 		/* receive result of peer, handle errors */
 
-		if(!datarecv(xbfr,BLOWFISH_BLOCKSIZE,anchor))RET("datarecv",15);
+		if(!datarecv(xbfr,-BLOWFISH_BLOCKSIZE,anchor))
+			RET("datarecv",15);
 
 		/* compare stored and received result, handle errors */
 
@@ -1738,7 +1963,8 @@ pwderr:		printf("Passwords don't match\n");
 	{
 		/* decrypt master secrets with old password */
 
-		blowfish_key_schedule(oldpass,strlen(oldpass),sched);
+		blowfish_key_schedule((unsigned char *)oldpass,strlen(oldpass),
+			sched);
 		memset(oldpass,0,128);
 		decrypt_cbc(secret+MASTER0,BLOWFISH_MAXKEY*2,sched,anchor->piv);
 		memset(sched,0,BLOWFISH_SCHEDULESIZE);
@@ -1750,7 +1976,8 @@ pwderr:		printf("Passwords don't match\n");
 	{
 		/* encrypt master secrets with new password */
 
-		blowfish_key_schedule(newpass,strlen(newpass),sched);
+		blowfish_key_schedule((unsigned char *)newpass,strlen(newpass),
+			sched);
 		memset(newpass,0,128);
 		encrypt_cbc(secret+MASTER0,BLOWFISH_MAXKEY*2,sched,anchor->piv);
 		memset(sched,0,BLOWFISH_SCHEDULESIZE);
