@@ -386,7 +386,7 @@ int chkip(char *ip)
 
 	/* success, if conversion to ip succeeds */
 
-	if(getaddr(ip)!=-1)return 1;
+	if(getaddr((unsigned char *)(ip))!=-1)return 1;
 
 	/* error, if not special case */
 
@@ -832,11 +832,13 @@ int parse(VPN *anchor)
 				"for '%s' command.\n",config[i].item);
 			return 0;
 		}
-		anchor->rtab[anchor->rmax][0]=getaddr(CFG_ROUTE(i,p1));
+		anchor->rtab[anchor->rmax][0]=
+			getaddr((unsigned char *)(CFG_ROUTE(i,p1)));
 		anchor->rtab[anchor->rmax][1]=
 			strcmp(CFG_ROUTE(i,p2),ip255)?
-				getaddr(CFG_ROUTE(i,p2)):-1;
-		anchor->rtab[anchor->rmax][2]=getaddr(CFG_ROUTE(i,p3));
+				getaddr((unsigned char *)(CFG_ROUTE(i,p2))):-1;
+		anchor->rtab[anchor->rmax][2]=
+			getaddr((unsigned char *)(CFG_ROUTE(i,p3)));
 		anchor->rmax++;
 	}
 
@@ -947,8 +949,10 @@ int parse(VPN *anchor)
 		/* in client mode peer is server ip,
 		   in server mode peer is client ip */
 
-		if(anchor->mode)anchor->peerip=getaddr(anchor->server_ip);
-		else anchor->peerip=getaddr(anchor->client_ip);
+		if(anchor->mode)anchor->peerip=
+			getaddr((unsigned char *)(anchor->server_ip));
+		else anchor->peerip=
+			getaddr((unsigned char *)(anchor->client_ip));
 
 		/* if a fixed device is given */
 
